#!/usr/bin/env python

# import standard libraries
from copy import deepcopy

# third party imports
import numpy as np

# local imports
from esi_utils_rupture.tensor import plane_to_tensor


def get_kagan_angle(strike1, dip1, rake1, strike2, dip2, rake2):
    """Calculate the Kagan angle between two moment tensors defined by strike,dip and
    rake.

    Kagan, Y. "Simplified algorithms for calculating double-couple rotation",
    Geophysical Journal, Volume 171, Issue 1, pp. 411-418.

    Args:
        strike1 (float): strike of slab or moment tensor
        dip1 (float): dip of slab or moment tensor
        rake1 (float): rake of slab or moment tensor
        strike2 (float): strike of slab or moment tensor
        dip2 (float): dip of slab or moment tensor
        rake2 (float): rake of slab or moment tensor
    Returns:
        float: Kagan angle between two moment tensors
    """
    # convert from strike, dip , rake to moment tensor
    tensor1 = plane_to_tensor(strike1, dip1, rake1)
    tensor2 = plane_to_tensor(strike2, dip2, rake2)

    kagan = calc_theta(tensor1, tensor2)

    return kagan


def calc_theta(vm1, vm2):
    """Calculate angle between two moment tensor matrices.

    Args:
        vm1 (ndarray): Moment Tensor matrix (see plane_to_tensor).
        vm2 (ndarray): Moment Tensor matrix (see plane_to_tensor).
    Returns:
        float: Kagan angle (degrees) between input moment tensors.
    """
    # calculate the eigenvectors of either moment tensor
    V1 = calc_eigenvec(vm1)
    V2 = calc_eigenvec(vm2)

    # find angle between rakes
    th = ang_from_R1R2(V1, V2)

    # calculate kagan angle and return
    for j in range(3):
        k = (j + 1) % 3
        V3 = deepcopy(V2)
        V3[:, j] = -V3[:, j]
        V3[:, k] = -V3[:, k]
        x = ang_from_R1R2(V1, V3)
        if x < th:
            th = x
    return th * 180.0 / np.pi


def calc_eigenvec(TM):
    """Calculate eigenvector of moment tensor matrix.


    Args:
        ndarray: moment tensor matrix (see plane_to_tensor)
    Returns:
        ndarray: eigenvector representation of input moment tensor.
    """

    # calculate eigenvector
    V, S = np.linalg.eigh(TM)
    inds = np.argsort(V)
    S = S[:, inds]
    S[:, 2] = np.cross(S[:, 0], S[:, 1])
    return S


def ang_from_R1R2(R1, R2):
    """Calculate angle between two eigenvectors.

    Args:
        R1 (ndarray): eigenvector of first moment tensor
        R2 (ndarray): eigenvector of second moment tensor
    Returns:
        float: angle between eigenvectors
    """
    return np.arccos(np.clip((np.trace(np.dot(R1, R2.transpose())) - 1.0) / 2.0, -1, 1))



#!/usr/bin/env python

# stdlib imports
import inspect
import logging
import pathlib
import sys
from enum import Enum
from io import StringIO
from typing import Iterable, Union

# third party imports
import click
import typer
from rich.console import group
from rich.markdown import Markdown
from rich.text import Text

# local imports
from strec.database import fetch_dataframe, read_datafile, stash_dataframe
from strec.gcmt import fetch_gcmt
from strec.slab import get_slab_grids
from strec.utils import create_config, get_config, get_config_file_name
from typer.core import MarkupMode
from typer.rich_utils import (
    MARKUP_MODE_MARKDOWN,
    STYLE_HELPTEXT_FIRST_LINE,
    _make_rich_rext,
)


# Monkey-patching as described here: https://github.com/tiangolo/typer/issues/447#issuecomment-1226458835
# This allows markdown help to work as expected.
@group()
def _get_custom_help_text(
    *,
    obj: Union[click.Command, click.Group],
    markup_mode: MarkupMode,
) -> Iterable[Union[Markdown, Text]]:
    # Fetch and dedent the help text
    help_text = inspect.cleandoc(obj.help or "")

    # Trim off anything that comes after \f on its own line
    help_text = help_text.partition("\f")[0]

    # Get the first paragraph
    first_line = help_text.split("\n\n")[0]
    # Remove single linebreaks
    if markup_mode != MARKUP_MODE_MARKDOWN and not first_line.startswith("\b"):
        first_line = first_line.replace("\n", " ")
    yield _make_rich_rext(
        text=first_line.strip(),
        style=STYLE_HELPTEXT_FIRST_LINE,
        markup_mode=markup_mode,
    )

    # Get remaining lines, remove single line breaks and format as dim
    remaining_paragraphs = help_text.split("\n\n")[1:]
    if remaining_paragraphs:
        remaining_lines = inspect.cleandoc(
            "\n\n".join(remaining_paragraphs).replace("<br/>", "\\")
        )
        yield _make_rich_rext(
            text=remaining_lines,
            style="cyan",
            markup_mode=markup_mode,
        )


typer.rich_utils._get_help_text = _get_custom_help_text

HELPSTR = """
Initialize the system with the Slab 2.0 grids and GCMT moment tensor database.

# Usage

`strec_cfg update --datafolder <path/to/data/folder> --slab --gcmt</code>`

For example, if you set the STREC data folder to be */data/strec*:

`strec_cfg update --datafolder /data/strec --slab --gcmt`

and then use the following command to see the resulting configuration:

`strec_cfg info`

The output should look something like the following:

```
Config file /home/user/.strec/config.ini:
------------------------
[DATA]
folder = /data/strec
slabfolder = /data/strec/slabs
dbfile = /data/strec/moment_tensors.db

[CONSTANTS]
minradial_disthist = 0.01
maxradial_disthist = 1.0
minradial_distcomp = 0.5
maxradial_distcomp = 1.0
step_distcomp = 0.1
depth_rangecomp = 10
minno_comp = 3
default_szdip = 17
dstrike_interf = 30
ddip_interf = 30
dlambda = 60
ddepth_interf = 20
ddepth_intra = 10
------------------------

Moment Tensor Database (/data/strec/moment_tensors.db) contains 60535 events from 1 sources.

There are 135 slab grids from 27 unique slab models located in /data/strec/slabs.
```


"""
app = typer.Typer(help=HELPSTR, rich_markup_mode="markdown")


# I hoped making an enum of string = loglevel would be clear, but typer prints the integer values
# of the log levels instead of CRITICAL, ERROR, etc. Making an enum and then a dict to make this
# clearer.
class LoggingLevel(str, Enum):
    CRITICAL = "CRITICAL"
    ERROR = "ERROR"
    WARNING = "WARNING"
    INFO = "INFO"
    DEBUG = "DEBUG"


LOGDICT = {
    "CRITICAL": logging.CRITICAL,
    "ERROR": logging.ERROR,
    "WARNING": logging.WARNING,
    "INFO": logging.INFO,
    "DEBUG": logging.DEBUG,
}


def config_exists():
    config_file = get_config_file_name()
    return config_file.exists()


def get_datafolder():
    try:
        config = get_config()
        datafolder = config["DATA"]["folder"]
        return datafolder
    except Exception:
        return None


@app.command()
def info():
    if not config_exists():
        print("No config file exists at. Run the update command.")
        sys.exit(1)
    config_file = get_config_file_name()
    print(f"Config file {config_file}:")
    print("------------------------")
    config = get_config()
    config_str = StringIO()
    config.write(config_str)
    config_str.seek(0)
    print(config_str.getvalue().rstrip())
    print("------------------------\n")
    dbfile = config["DATA"]["dbfile"]
    dataframe = fetch_dataframe(dbfile)
    nsources = len(dataframe["source"].unique())
    print(
        (
            f"Moment Tensor Database ({dbfile}) contains {len(dataframe)} "
            f"events from {nsources} sources.\n"
        )
    )
    slabfolder = config["DATA"]["slabfolder"]
    slab_files = list(pathlib.Path(slabfolder).glob("*.grd"))
    names = [slab_file.name for slab_file in slab_files]
    slabs = set([name.split("_")[0] for name in names])
    print(
        (
            f"There are {len(slab_files)} slab grids from {len(slabs)} "
            f"unique slab models located in {slabfolder}."
        )
    )
    sys.exit(0)


@app.command()
def update(
    datafolder: str = typer.Option(
        get_datafolder,
        help=(
            "Folder where slab data and moment tensor data will be stored. "
            "Defaults to previously configured value."
        ),
    ),
    slab: bool = typer.Option(False, help="Download slab data"),
    gcmt: bool = typer.Option(False, help="Download GCMT moment tensor data"),
    moment_data: str = typer.Option(None, help="Supply moment data as CSV/Excel"),
    moment_data_source: str = typer.Option(
        "unknown", help="Supply moment data source (US, CI, etc.)"
    ),
    log: LoggingLevel = typer.Option(
        "INFO",
        help=(
            "Set logging level (https://docs.python.org/3/"
            "library/logging.html#logging-levels)"
        ),
    ),
):
    if log:
        logging.basicConfig(level=LOGDICT[log])
    if datafolder is None:
        print("Config file does not exist. Please specify a datafolder.")
        sys.exit(1)
    if gcmt and (moment_data is not None or moment_data_source != "unknown"):
        print(
            (
                "You may either choose to download GCMT "
                "moment tensors or install your own source, not both."
            )
        )
        sys.exit(1)
    datafolder = pathlib.Path(datafolder)
    if not config_exists():
        config = create_config(datafolder)
    else:
        config = get_config()
        if str(datafolder) != config["DATA"]["folder"]:
            # TODO - update or raise error?
            print("datafolder is already configured. Exiting.")
            sys.exit(1)

    messages = []
    if slab:
        slabfolder = pathlib.Path(config["DATA"]["slabfolder"])
        if not slabfolder.exists():
            slabfolder.mkdir(parents=True)
        slab_result, slab_msg = get_slab_grids(slabfolder)
        if not slab_result:
            messages.append(slab_msg)
    if gcmt:
        try:
            gcmt_dataframe = fetch_gcmt()
            dbfile = config["DATA"]["dbfile"]
            source = "GCMT"
            stash_dataframe(gcmt_dataframe, dbfile, source, create_db=True)
        except Exception as e:
            gcmt_msg = f"Failed to download GCMT data: {str(e)}"
            messages.append(gcmt_msg)

    if moment_data:
        dbfile = pathlib.Path(config["DATA"]["dbfile"])
        create_db = True
        if dbfile.exists():
            create_db = False
        try:
            moment_dataframe = read_datafile(moment_data)
            stash_dataframe(
                moment_dataframe, dbfile, moment_data_source, create_db=create_db
            )
        except Exception as e:
            moment_msg = f"Could not parse moment datafile {moment_data}: {str(e)}"
            messages.append(moment_msg)
    if len(messages):
        print("Errors were encountered during the course of downloading/loading data:")
        for message in messages:
            print(f"'{message}'")
    sys.exit(0)


if __name__ == "__main__":
    app()



#!/usr/bin/env python

# stdlib imports
import sys
from enum import Enum
from typing import Tuple

# third party imports
import typer

# local imports
from strec.calc import select_regions
from strec.utils import render_row

app = typer.Typer(rich_markup_mode="rich")


class FormatType(str, Enum):
    pretty = "pretty"
    csv = "csv"
    excel = "excel"
    json = "json"


@app.command()
def main(
    eqinfo: Tuple[float, float, float, float] = typer.Option(
        (None, None, None, None),
        "--eqinfo",
        "-e",
        help=("Calculate region based on lat, lon, depth, magnitude of earthquake"),
    ),
    event_id: str = typer.Option(
        None,
        "--event-id",
        "-d",
        help=("Calculate region based on ComCat event id (i.e., us6000iasi)."),
    ),
    input_file: str = typer.Option(
        None,
        "--input-file",
        "-i",
        help="Calculate region for events specified in input file.",
    ),
    hypo_columns: Tuple[str, str, str, str] = typer.Option(
        (None, None, None, None),
        "--hypo_columns",
        "-c",
        help=(
            (
                "When used with -i/--input-file, specify the columns "
                "that should be used for Latitude, Longitude, Depth and Magnitude."
            )
        ),
    ),
    id_column: str = typer.Option(
        None,
        "--id-column",
        help=(
            (
                "When used with -i/--input-file, specify the column "
                "that should be used for ComCatID."
            )
        ),
    ),
    format: FormatType = typer.Option(
        FormatType.pretty,
        "--format",
        "-f",
        help="Specify output format for results.",
    ),
    output_file: str = typer.Option(
        None,
        "--output-file",
        "-o",
        help="Specify output filename for results (see --format).",
    ),
    moment_info: Tuple[float, float, float] = typer.Option(
        (None, None, None),
        "--moment-info",
        "-m",
        help="Specify strike, dip, rake for single earthquake.",
    ),
    verbose: bool = typer.Option(
        False,
        "--verbose",
        "-v",
        help="Print out more verbose output.",
    ),
):
    """Determine various seismo-tectonic parameters for given input coordinates.

    The output will consist of the following data fields:

     - TectonicRegion : (Active, Stable, Volcanic, Subduction)
     - FocalMechanism : (RS [Reverse],SS [Strike-Slip], NM [Normal], ALL)
     - TensorType : composite
     - TensorSource : (composite, one of many moment tensor sources like GCMT)
     - KaganAngle : Angle between fault plane and slab orientation.
     - CompositeVariability :  A measure of the uncertainty in the composite
                               moment tensor.
     - NComposite : Number of events used to create composite moment tensor.
     - DistanceToStable : Distance in km from the nearest stable polygon.
     - DistanceToActive : Distance in km from the nearest active polygon.
     - DistanceToSubduction : Distance in km from the nearest subduction
                              polygon.
     - DistanceToVolcanic : Distance in km from the nearest volcanic polygon.
     - Oceanic : Boolean indicating whether we are in an oceanic region,
                 false indicates Continental.
     - DistanceToOceanic : Distance in km to nearest oceanic polygon.
     - DistanceToContinental : Distance in km to nearest continental polygon.
     - SlabModelRegion : Subduction region.
     - SlabModelDepth : Depth of slab2.0 grid at earthquake location.
     - SlabModelDepthUncertainty : Uncertainty of depth to slab.
     - SlabModelDip : Dip of slab at earthquake location.
     - SlabModelStrike : Strike of slab at earthquake location.
     - SlabModelMaximumDepth : Maximum depth of seismogenic zone.
     - ProbabilityActive: Probability that event occurred in an active region.
     - ProbabilityStable: Probability that event occurred in an stable region.
     - ProbabilityVolcanic: Probability that event occurred in a volcanic region.
     - ProbabilitySubduction: Probability that event occurred in a subduction region.
     - ProbabilityCrustal: Probability that event occurred in a subduction
                           crustal depth zone.
     - ProbabilityInterface: Probability that event occurred in a subduction
                             interface depth zone.
     - ProbabilityIntraslab: Probability that event occurred in a subduction
                             intraslab depth zone.
     - ProbabilityActiveShallow: When specified, the probability that event occurred
                                 in a configurable shallow zone in an active tectonic
                                 region.
     - ProbabilityActiveDeep: When specified, the probability that event occurred
                              in a configurable deep zone in an active tectonic
                              region.
     - ProbabilityStableShallow: When specified, the probability that event occurred
                                 in a configurable shallow zone in a stable tectonic
                                 region.
     - ProbabilityVolcanicShallow: When specified, the probability that event occurred
                                   in a configurable shallow zone in a volcanic tectonic
                                   region.

    where:
        - Active regions are boundaries between two plates that are colliding,
          sliding past each other, or pulling apart.
        - Stable regions are regions where those things are not happening.
        - Subduction regions are those that are above the Slab2.0 grids.
        - Volcanic regions are those that sit above mantle plumes
        - ALL FocalMechanisms that are some combination of the other types
        -


    Input files can be CSV or Excel format (regcalc will
    attempt to determine format automatically) and MUST include columns
    beginning with (case insensitive):

        Lat - Numeric latitude of input earthquake.
        Lon - Numeric longitude of input earthquake.
        Depth - Numeric depth of earthquake (km).

    Input files may optionally contain a column (also case insensitive) called:
    ComCatID - Valid ANSS ComCat ID
    Any other columns present in the input will be copied to the output.

    The output format must be one of the following:

        pretty - A human readable pretty-printed representation of the output.
        json - A JSON representation of the output
        csv - A CSV output, one earthquake per line.
        excel - Microsoft Excel format, one earthquake per line.

    Note that the -c/--columns option will override these defaults.
    """
    # error out on any invalid combinations of options
    if format in ["pretty", "json"] and output_file is not None:
        print(f"Output file is invalid when specified with format of {format}")
        sys.exit(1)

    if format in ["excel"] and output_file is None:
        print("Output format 'excel' is invalid unless an output file is specified")
        sys.exit(1)

    has_eq = int(eqinfo[0] is not None)
    has_id = int(event_id is not None)
    has_file = int(input_file is not None)
    if sum([has_eq, has_id, has_file]) > 1:
        print(
            "Options --eqinfo, --event-id, --input-file are "
            "mutually exclusive. Please choose one."
        )
        sys.exit(1)

    if (hypo_columns[0] is not None or id_column is not None) and input_file is None:
        print(
            "Options --hypo-columns and --id-column only work "
            "when input_file is specified."
        )
        sys.exit(1)

    if moment_info[0] is not None and input_file is not None:
        print("You can only specify moment information for a single earthquake.")
        sys.exit(1)

    regions_frame = select_regions(
        input_file,
        eqinfo,
        moment_info,
        event_id,
        hypo_columns,
        id_column,
        verbose,
    )
    if output_file is None:
        for idx, row in regions_frame.iterrows():
            render_row(row, format, hypo_columns)

    if output_file:
        if format == "csv":
            regions_frame.to_csv(output_file, index=False)
        else:
            regions_frame.to_excel(output_file, index=False)


if __name__ == "__main__":
    app()



# stdlib imports
import logging

# third party imports
import numpy as np
import pandas as pd
from esi_utils_rupture.tensor import (
    fill_tensor_from_angles,
    fill_tensor_from_components,
)

# local imports
from strec.subtype import SubductionSelector, get_event_details
from strec.utils import read_input_file

LOGGER = "subselect"


def get_input_dataframe(input_file, eqinfo, event_id, hypo_columns, id_column):
    """Create a dataframe of events (possibly only one).

    Args:
        input_file (str or None):  Name of input CSV/Excel file or None.
        eqinfo (tuple or None): Tuple of (lat, lon, depth, magnitude) or None.
        event_id (str or None): ComCat Event ID or None.
        hypo_columns (tuple or None): Tuple of hypocenter column names in input_file or
                                      None.
        id_column (str or None): Name of column in input_file containing ComCat Event
                                 ID or None.

    Returns:
        tuple of:
            - pandas DataFrame containing at least lat/lon/depth columns
            - Name of ComCat event ID column or None
            - Name of latitude column
            - Name of longitude column
            - Name of depth column
    """
    latcol = "lat"
    loncol = "lon"
    depcol = "depth"
    magcol = "mag"
    idcol = None
    if input_file is not None:
        df, idcol, latcol, loncol, depcol, magcol = read_input_file(
            input_file, hypo_columns, id_column
        )
    elif eqinfo is not None:
        lat, lon, depth, mag = eqinfo
        d = {"lat": [lat], "lon": [lon], "depth": [depth], "mag": [mag]}
        df = pd.DataFrame(d)
    if event_id is not None:
        detail = get_event_details(event_id)
        idcol = "ComCatID"
        lat, lon, depth, mag = (
            detail["latitude"],
            detail["longitude"],
            detail["depth"],
            detail["magnitude"],
        )
        d = {
            "ComCatID": event_id,
            "lat": [lat],
            "lon": [lon],
            "depth": [depth],
            "mag": [mag],
        }
        df = pd.DataFrame(d)
    return (df, idcol, latcol, loncol, depcol, magcol)


def get_moment_columns(row):
    """Return focal mechanism/moment tensor parameters from MT components or focal angles.

    Args:
        row (pandas Series): Contains moment tensor components (mtt, mpp, etc) OR
                             focal angles (strike, dip, rake)

    Returns:
        dict: Fully descriptive moment tensor dictionary, including fields:
            - mrr,mtt,mpp,mrt,mrp,mtp Moment tensor components.
            - T T-axis values:
              - azimuth (degrees)
              - plunge (degrees)
            - N N-axis values:
              - azimuth (degrees)
              - plunge (degrees)
            - P P-axis values:
              - azimuth (degrees)
              - plunge (degrees)
            - NP1 First nodal plane values:
              - strike (degrees)
              - dip (degrees)
              - rake (degrees)
            - NP2 Second nodal plane values:
              - strike (degrees)
              - dip (degrees)
              - rake (degrees)
    """
    # row is a pandas series object
    isreal = row.notnull()

    has_mtt = row.index.str.contains("mtt", case=False)
    has_mpp = row.index.str.contains("mpp", case=False)
    has_mrr = row.index.str.contains("mrr", case=False)
    has_mrt = row.index.str.contains("mrt", case=False)
    has_mrp = row.index.str.contains("mrp", case=False)
    has_mtp = row.index.str.contains("mtp", case=False)

    mtts = row[isreal & has_mtt]
    mpps = row[isreal & has_mpp]
    mrrs = row[isreal & has_mrr]
    mrts = row[isreal & has_mrt]
    mrps = row[isreal & has_mrp]
    mtps = row[isreal & has_mtp]

    mtt = mpp = mrr = mrt = mrp = mtp = np.nan
    if len(mtts):
        mtt = float(mtts[0])
    if len(mpps):
        mpp = float(mpps[0])
    if len(mrrs):
        mrr = float(mrrs[0])
    if len(mrts):
        mrt = float(mrts[0])
    if len(mrps):
        mrp = float(mrps[0])
    if len(mtps):
        mtp = float(mtps[0])

    if not np.isnan(np.sum([mtt, mpp, mrr, mrt, mrp, mtp])):
        tensor_params = fill_tensor_from_components(mrr, mtt, mpp, mrt, mrp, mtp)
        return tensor_params

    has_strike = row.index.str.contains("strike", case=False)
    has_dip = row.index.str.contains("dip", case=False)
    has_rake = row.index.str.contains("rake", case=False)
    strikes = row[isreal & has_strike]
    dips = row[isreal & has_dip]
    rakes = row[isreal & has_rake]

    strike = dip = rake = np.nan
    if len(strikes):
        strike = strikes[0]
    if len(dips):
        dip = dips[0]
    if len(rakes):
        rake = rakes[0]

    if not np.isnan(strike):
        tensor_params = fill_tensor_from_angles(strike, dip, rake)
        return tensor_params

    return None


def check_moment_row(row):
    """Check row of dataframe for presence of moment tensor information.

    Args:
        row (pandas Series): Series possibly containing moment tensor data
    Returns:
        bool: True if input contains moment tensor or focal mechanism data.
    """
    # row is a pandas series object
    hasAngles = True
    hasComponents = True

    keys = set([key.lower() for key in (row.keys()).values.tolist()])
    cmp_components = set(["mrr", "mpp", "mtt", "mrt", "mrp", "mtp"])
    if not cmp_components.issubset(keys):
        hasComponents = False

    cmp_angles = set(["strike", "dip", "rake"])
    if not cmp_angles.issubset(keys):
        hasAngles = False

    return hasAngles or hasComponents


def select_regions(
    input_file,
    eqinfo,
    moment_info,
    event_id,
    hypo_columns,
    id_column,
    verbose,
):
    """Output full range of seismotectonic information.

    Args:
        input_file (str or None):  Name of input CSV/Excel file or None.
        eqinfo (tuple or None): Tuple of (lat, lon, depth, magnitude) or None.
        moment_info (tuple or None): Tuple of (strike, dip, rake) or None.
        event_id (str or None): ComCat Event ID or None.
        hypo_columns (tuple or None): Tuple of hypocenter column names in input_file
                                      or None.
        id_column (str or None): Name of column in input_file containing ComCat
                                 Event ID or None.
        verbose (bool): Turn verbose debugging on/off.
    Returns:
        Pandas Series object with indices:
            - TectonicRegion : (Subduction,Active,Stable,Volcanic)
            - FocalMechanism : (RS [Reverse],SS [Strike-Slip], NM [Normal], ALL
            [Unknown])
            - TensorType : (actual, composite)
            - TensorSource : String indicating the source of the moment tensor
            information.
            - KaganAngle : Angle between moment tensor and slab interface.
            - CompositeVariability : A measure of the uncertainty in the composite
            moment tensor.
            - NComposite : Number of events used to create composite moment tensor.
            - DistanceToStable : Distance in km from the nearest stable polygon.
            - DistanceToActive : Distance in km from the nearest active polygon.
            - DistanceToSubduction : Distance in km from the nearest subduction
            polygon.
            - DistanceToVolcanic : Distance in km from the nearest volcanic polygon.
            - Oceanic : Boolean indicating whether we are in an oceanic region.
            - DistanceToOceanic : Distance in km to nearest oceanic polygon.
            - DistanceToContinental : Distance in km to nearest continental polygon.
            - SlabModelRegion : Subduction region.
            - SlabModelType : (grid,trench)
            - SlabModelDepth : Depth to slab interface at epicenter.
            - SlabModelDepthUncertainty : Uncertainty of depth to slab interface.
            - SlabModelDip : Dip of slab at epicenter.
            - SlabModelStrike : Strike of slab at epicenter.
    """
    selector = SubductionSelector(verbose=verbose, prefix=LOGGER)
    dataframe, idcol, latcol, loncol, depcol, magcol = get_input_dataframe(
        input_file, eqinfo, event_id, hypo_columns, id_column
    )
    tensor_params = None
    if moment_info[0] is not None:
        strike, dip, rake = moment_info
        tensor_params = fill_tensor_from_angles(strike, dip, rake, dataframe[magcol])
    has_tensor = check_moment_row(dataframe.iloc[0]) or tensor_params is not None
    rows = []
    ic = 0
    inc = min(100, np.power(10, np.floor(np.log10(len(dataframe))) - 1))
    for _, row in dataframe.iterrows():
        if ic % inc == 0 and verbose:
            msg = "Getting detailed information for %i of %i events.\n"
            logging.info(msg % (ic, len(dataframe)))
        if idcol is not None:
            event_id = row[idcol]
        if not tensor_params and has_tensor:
            tensor_params = get_moment_columns(row)

        lat = row[latcol]
        lon = row[loncol]
        # for the cases where lon goes from 0-360
        if lon > 180:
            lon -= 360
        depth = row[depcol]
        mag = row[magcol]
        result = selector.getSubductionType(
            lat,
            lon,
            depth,
            mag,
            eventid=event_id,
            tensor_params=tensor_params,
        )

        tensor_params = None
        row = pd.concat([row, result])
        rows.append(row)
    output_dataframe = pd.DataFrame(rows)
    return output_dataframe
